PROGRAM Compute_dlnPkdz3PT
  USE cosmo
  USE growth
  USE nonlinearpk
  ! A program for computing dlnPk/dz using the 3rd-order perturbation 
  ! theory (3PT). It will compute dlnPk/dz in two different ways:
  ! (1) Numerically differentiating P(k,z) from "generate_pkz3pt" using
  !     a spline interpolation.
  ! (2) Calculating dlnPk/dz exactly from the formula of the 3PT P(k,z).
  ! This code therefore allows one to check the accuracy of the numerical
  ! differentiation using a spline interpolation.
  ! - k is in units of h Mpc^-1
  ! - P(k) is in units of h^-3 Mpc^3
  ! January 2, 2012: E.Komatsu
  IMPLICIT none
  character(len=128) :: filename
  integer :: nz=901 ! # of data points in redshift in the input data file
  integer :: nk=691 ! # of data points in wavenumber in the input data file
  double precision :: zmax=9d0 ! maximum redshift in the input data file
  double precision, allocatable, dimension(:,:) :: pkz,y2a
  double precision, allocatable, dimension(:) :: redshift,k_ov_h
  double precision :: dlnPkdz,lnPk,kout_ov_h,zout,dln1pz
  integer :: ik,iz,n
  double precision :: g,dgdlna,dz,D,dDdz
  double precision :: nonlinear_dlnpkddz,zin
  external g,dgdlna,nonlinear_dlnpkddz
! Specify three cosmological parameters
! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  w=-1d0
! tabulate g(z) by calling "setup_growth"
  call setup_growth
! ask for the wavenumber
  print*,'wavenumber [h Mpc^-1]?'
  read*,kout_ov_h
  if(kout_ov_h<1d-4)then
     print*,kout_ov_h,'is smaller than 1e-4 Mpc/h.'
     stop
  else if(kout_ov_h>1d2)then
     print*,kout_ov_h,'is greater than 1e2 Mpc/h.'
     stop
  endif
! read in and tabulate P(k,z)
  ALLOCATE(pkz(nz,nk),y2a(nz,nk),redshift(nz),k_ov_h(nk))
  filename='redshift_wavenumber_pk3pt.txt'
  print*,'read in '//trim(filename) &
       //' and generate an interpolation table'
  open(1,file=filename,status='old')
  do iz=1,nz
     do ik=1,nk
        read(1,*)redshift(iz),k_ov_h(ik),pkz(iz,ik)
     enddo
  enddo
  close(1)
  CALL splie2(redshift,dlog(k_ov_h),dlog(pkz),nz,nk,y2a)
  filename='redshift_dlnpkdz3ptnumerical_dlnpkdz3ptanalytical.txt'
  open(2,file=filename,status='unknown')
! read in and tabulate P(k)
  filename='wavenumber_pkd11_pkd22_pkd13_at_z=30.txt'
  n=691    ! # of lines in the file
  zin=30d0 ! redshift of the input power spectrum
  dln1pz=dlog((1d0+zmax))/1d2 ! dlog(1+zmax)/(# of points in redshift)
  do iz=1,100
     zout=dexp(dble(iz)*dln1pz)-1d0
! numerical dlnpkddz from a spline interpolation
     CALL splin2_der1(redshift,dlog(k_ov_h),dlog(pkz),y2a,nz,nk, &
          zout,dlog(kout_ov_h),dlnPkdz)
! analytical dlnpkddz from 3PT formula
     D=g(zout)/g(zin)*(1d0+zin)/(1d0+zout) ! linear growth relative to zin
     dDdz=-D*(1d0+dgdlna(zout)/g(zout))/(1d0+zout)
     CALL open_nonlineardlnpkddz(filename,n,D,dDdz)
! output results
     write(2,'(3E18.8)')zout,dlnPkdz,nonlinear_dlnpkddz(kout_ov_h)
     CALL close_nonlineardlnpkddz
  enddo
  close(2)
  DEALLOCATE(pkz,y2a,redshift,k_ov_h)
END PROGRAM Compute_dlnPkdz3PT
